/*==========================================================================
 *
 *  Copyright (C) 1996 Microsoft Corporation.  All Rights Reserved.
 *
 *  File:       dialog.cpp
 *  Content:	Creates a dialog to query the user for connection settings
 *				and establish a connection.
 *@@BEGIN_MSINTERNAL
 *  History:
 *   Date	By	Reason
 *   ====	==	======
 *	7/12/96	kipo	created it
 * 10/23/96	kipo	updated to include review feedback
 *@@END_MSINTERNAL
 *
 ***************************************************************************/

#define INITGUID
#include <windows.h>
#include <windowsx.h>

#include "dpchat.h"
#include "resource.h"

// constants
const DWORD MAXNAMELEN = 200;			// max size of a session or player name

// prototypes
BOOL CALLBACK	ConnectWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

BOOL FAR PASCAL DirectPlayEnumerateCallback(LPGUID lpSPGuid, LPTSTR lpszSPName,
							DWORD dwMajorVersion, DWORD dwMinorVersion, LPVOID lpContext);

HRESULT			CreateDirectPlayInterface(LPGUID lpguidServiceProvider,
							LPDIRECTPLAY2A *lplpDirectPlay2A);
HRESULT			DestroyDirectPlayInterface(HWND hWnd, LPDIRECTPLAY2A lpDirectPlay2A);
HRESULT			HostSession(LPDIRECTPLAY2A lpDirectPlay2A,
							LPSTR lpszSessionName, LPSTR lpszPlayerName,
							LPDPLAYINFO lpDPInfo);
HRESULT			JoinSession(LPDIRECTPLAY2A lpDirectPlay2A,
							LPGUID lpguidSessionInstance, LPSTR lpszPlayerName,
							LPDPLAYINFO lpDPInfo);
HRESULT			EnumSessions(HWND hWnd, LPDIRECTPLAY2A lpDirectPlay2A);

HRESULT			GetServiceProviderGuid(HWND hWnd, LPGUID lpguidServiceProvider);
void			DeleteServiceProviderList(HWND hWnd);
HRESULT			GetSessionInstanceGuid(HWND hWnd, LPGUID lpguidSessionInstance);
void			DeleteSessionInstanceList(HWND hWnd);
void			EnableDlgButton(HWND hDlg, int nIDDlgItem, BOOL bEnable);

HRESULT ConnectUsingDialog(HINSTANCE hInstance, LPDPLAYINFO lpDPInfo)
{
	// ask user for connection settings
	if (DialogBoxParam(hInstance, MAKEINTRESOURCE(IDD_CONNECTDIALOG),
					   NULL, (DLGPROC) ConnectWndProc, (LPARAM) lpDPInfo))
	{
		return (DP_OK);
	}
	else
	{
		return (DPERR_USERCANCEL);
	}
}

BOOL CALLBACK ConnectWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	static LPDPLAYINFO		lpDPInfo;
	static LPDIRECTPLAY2A	lpDirectPlay2A;
	GUID					guidServiceProvider;
	GUID					guidSessionInstance;
	char					szSessionName[MAXNAMELEN];
	char					szPlayerName[MAXNAMELEN];
	DWORD					dwNameSize;
	HRESULT					hr;

    switch(uMsg)
    {
    case WM_INITDIALOG:
        // save the connection info pointer
        lpDPInfo = (LPDPLAYINFO) lParam;
		lpDirectPlay2A = NULL;

		// put all the service providers in a combo box
		DirectPlayEnumerate(DirectPlayEnumerateCallback, hWnd);
		SendDlgItemMessage(hWnd, IDC_SPCOMBO, CB_SETCURSEL, (WPARAM) 0, (LPARAM) 0);

		// setup initial button state
		EnableDlgButton(hWnd, IDC_HOSTBUTTON, TRUE);
		EnableDlgButton(hWnd, IDC_JOINBUTTON, FALSE);
		EnableDlgButton(hWnd, IDC_ENUMERATEBUTTON, TRUE);
        break;

    case WM_DESTROY:
		// delete information stored along with the lists
		DeleteServiceProviderList(hWnd);
		DeleteSessionInstanceList(hWnd);
        break;

    case WM_COMMAND:
        switch(LOWORD(wParam))
        {
		case IDC_SPCOMBO:
			switch (HIWORD(wParam))
			{
			case CBN_SELCHANGE:
				// service provider changed, so rebuild display and
				// delete any existing DirectPlay interface
				hr = DestroyDirectPlayInterface(hWnd, lpDirectPlay2A);
				lpDirectPlay2A = NULL;
				break;
			}
			break;

		case IDC_HOSTBUTTON:

			// DirectPlay interface has not been created yet
			if (lpDirectPlay2A == NULL)
			{
				// get guid of selected service provider
				hr = GetServiceProviderGuid(hWnd, &guidServiceProvider);
				if FAILED(hr)
					goto HOST_FAILURE;

				// create a DirectPlay interface using this service provider,
				hr = CreateDirectPlayInterface(&guidServiceProvider, &lpDirectPlay2A);
				if FAILED(hr)
					goto HOST_FAILURE;
			}

			// use computer name for session name
			dwNameSize = MAXNAMELEN;
			if (!GetComputerName(szSessionName, &dwNameSize))
				lstrcpy(szSessionName, "Session");

			// use user name for player name
			dwNameSize = MAXNAMELEN;
			if (!GetUserName(szPlayerName, &dwNameSize))
				lstrcpy(szPlayerName, "unknown");

			// host a new session on this service provider
			hr = HostSession(lpDirectPlay2A, szSessionName, szPlayerName, lpDPInfo);
			if FAILED(hr)
				goto HOST_FAILURE;

			// dismiss dialog if we succeeded in hosting
			EndDialog(hWnd, TRUE);
			break;

		HOST_FAILURE:
			ErrorBox("Could not host session because of error 0x%08X", hr);

			hr = DestroyDirectPlayInterface(hWnd, lpDirectPlay2A);
			lpDirectPlay2A = NULL;
			break;

        case IDC_JOINBUTTON:

			// should have an interface by now
			if (lpDirectPlay2A == NULL)
				break;

			// get guid of selected session instance
			hr = GetSessionInstanceGuid(hWnd, &guidSessionInstance);
			if FAILED(hr)
				goto JOIN_FAILURE;

			// use user name for player name
			dwNameSize = MAXNAMELEN;
			if (!GetUserName(szPlayerName, &dwNameSize))
				lstrcpy(szPlayerName, "unknown");

			// join this session
			hr = JoinSession(lpDirectPlay2A, &guidSessionInstance, szPlayerName, lpDPInfo);
			if FAILED(hr)
				goto JOIN_FAILURE;

			// dismiss dialog if we succeeded in joining
			EndDialog(hWnd, TRUE);
            break;

		JOIN_FAILURE:
			ErrorBox("Could not join session because of error 0x%08X", hr);

			hr = DestroyDirectPlayInterface(hWnd, lpDirectPlay2A);
			lpDirectPlay2A = NULL;
			break;

        case IDC_ENUMERATEBUTTON:

			// DirectPlay interface has not been created yet
			if (lpDirectPlay2A == NULL)
			{
				// get guid of selected service provider
				hr = GetServiceProviderGuid(hWnd, &guidServiceProvider);
				if FAILED(hr)
					goto ENUMERATE_FAILURE;

				// create a DirectPlay interface using this service provider,
				hr = CreateDirectPlayInterface(&guidServiceProvider, &lpDirectPlay2A);
				if FAILED(hr)
					goto ENUMERATE_FAILURE;
			}

			// dim button while enumerating
			EnableDlgButton(hWnd, IDC_ENUMERATEBUTTON, FALSE);

			// enumerate sessions on this service provider
			hr = EnumSessions(hWnd, lpDirectPlay2A);

			EnableDlgButton(hWnd, IDC_ENUMERATEBUTTON, TRUE);

			if FAILED(hr)
				goto ENUMERATE_FAILURE;

            break;

		ENUMERATE_FAILURE:
			ErrorBox("Could not enumerate sessions because of error 0x%08X", hr);

			hr = DestroyDirectPlayInterface(hWnd, lpDirectPlay2A);
			lpDirectPlay2A = NULL;
			break;

        case IDCANCEL:
			// delete any interface created if cancelling
			hr = DestroyDirectPlayInterface(hWnd, lpDirectPlay2A);
			lpDirectPlay2A = NULL;

			EndDialog(hWnd, FALSE);
            break;
        }

        break;
    }

    // Allow for default processing
    return FALSE;
}

BOOL FAR PASCAL DirectPlayEnumerateCallback(
						LPGUID		lpSPGuid,
						LPTSTR		lpszSPName,
						DWORD		dwMajorVersion,
						DWORD		dwMinorVersion,
						LPVOID		lpContext)
{
    HWND			hWnd = (HWND) lpContext;
    LRESULT			iIndex;
	LPGUID			lpGuid;

	// store service provider name in combo box
	iIndex = SendDlgItemMessage(hWnd, IDC_SPCOMBO, CB_ADDSTRING, 0, (LPARAM) lpszSPName);
	if (iIndex == CB_ERR)
		goto FAILURE;

	// make space for application GUID
	lpGuid = (LPGUID) GlobalAllocPtr(GHND, sizeof(GUID));
	if (lpGuid == NULL)
		goto FAILURE;

	// store pointer to GUID in combo box
	*lpGuid = *lpSPGuid;
	SendDlgItemMessage(hWnd, IDC_SPCOMBO, CB_SETITEMDATA, (WPARAM) iIndex, (LPARAM) lpGuid);

FAILURE:
    return (TRUE);
}

HRESULT CreateDirectPlayInterface(LPGUID lpguidServiceProvider,
								  LPDIRECTPLAY2A *lplpDirectPlay2A)
{
	LPDIRECTPLAY		lpDirectPlay1 = NULL;
	LPDIRECTPLAY2A		lpDirectPlay2A = NULL;
	HRESULT				hr;

	// get a DirectPlay 1.0 interface
	hr = DirectPlayCreate(lpguidServiceProvider, &lpDirectPlay1, NULL);
	if FAILED(hr)
		goto FAILURE;

	// query for an ANSI DirectPlay2 interface
	hr = lpDirectPlay1->QueryInterface(IID_IDirectPlay2A, (LPVOID *) &lpDirectPlay2A);
	if FAILED(hr)
		goto FAILURE;

	// return interface created
	*lplpDirectPlay2A = lpDirectPlay2A;

FAILURE:
	if (lpDirectPlay1)
		lpDirectPlay1->Release();

	return (hr);
}

HRESULT DestroyDirectPlayInterface(HWND hWnd, LPDIRECTPLAY2A lpDirectPlay2A)
{
	HRESULT		hr = DP_OK;

	if (lpDirectPlay2A)
	{
		DeleteSessionInstanceList(hWnd);
		EnableDlgButton(hWnd, IDC_JOINBUTTON, FALSE);

		hr = lpDirectPlay2A->Release();
	}

	return (hr);
}

HRESULT HostSession(LPDIRECTPLAY2A lpDirectPlay2A,
					LPSTR lpszSessionName, LPSTR lpszPlayerName,
					LPDPLAYINFO lpDPInfo)
{
	DPID				dpidPlayer;
	DPNAME				dpName;
	DPSESSIONDESC2		sessionDesc;
	HRESULT				hr;

	// check for valid interface
	if (lpDirectPlay2A == NULL)
		return (DPERR_INVALIDOBJECT);

	// host a new session
	ZeroMemory(&sessionDesc, sizeof(DPSESSIONDESC2));
	sessionDesc.dwSize = sizeof(DPSESSIONDESC2);
    sessionDesc.dwFlags = DPSESSION_MIGRATEHOST | DPSESSION_KEEPALIVE;
    sessionDesc.guidApplication = DPCHAT_GUID;
    sessionDesc.dwMaxPlayers = MAXPLAYERS;
	sessionDesc.lpszSessionNameA = lpszSessionName;

	hr = lpDirectPlay2A->Open(&sessionDesc, DPOPEN_CREATE);
	if FAILED(hr)
		goto OPEN_FAILURE;

	// fill out name structure
	ZeroMemory(&dpName, sizeof(DPNAME));
	dpName.dwSize = sizeof(DPNAME);
	dpName.lpszShortNameA = lpszPlayerName;
	dpName.lpszLongNameA = NULL;

	// create a player with this name
	hr = lpDirectPlay2A->CreatePlayer(&dpidPlayer, &dpName, 
							lpDPInfo->hPlayerEvent, NULL, 0, 0);
	if FAILED(hr)
		goto CREATEPLAYER_FAILURE;

	// return connection info
	lpDPInfo->lpDirectPlay2A = lpDirectPlay2A;
	lpDPInfo->dpidPlayer = dpidPlayer;
	lpDPInfo->bIsHost = TRUE;

	return (DP_OK);

CREATEPLAYER_FAILURE:
OPEN_FAILURE:
	lpDirectPlay2A->Close();
	return (hr);
}

HRESULT JoinSession(LPDIRECTPLAY2A lpDirectPlay2A,
					LPGUID lpguidSessionInstance, LPSTR lpszPlayerName,
					LPDPLAYINFO lpDPInfo)
{
	DPID				dpidPlayer;
	DPNAME				dpName;
	DPSESSIONDESC2		sessionDesc;
	HRESULT				hr;

	// check for valid interface
	if (lpDirectPlay2A == NULL)
		return (DPERR_INVALIDOBJECT);

	// join existing session
	ZeroMemory(&sessionDesc, sizeof(DPSESSIONDESC2));
	sessionDesc.dwSize = sizeof(DPSESSIONDESC2);
    sessionDesc.guidInstance = *lpguidSessionInstance;

	hr = lpDirectPlay2A->Open(&sessionDesc, DPOPEN_JOIN);
	if FAILED(hr)
		goto OPEN_FAILURE;

	// fill out name structure
	ZeroMemory(&dpName, sizeof(DPNAME));
	dpName.dwSize = sizeof(DPNAME);
	dpName.lpszShortNameA = lpszPlayerName;
	dpName.lpszLongNameA = NULL;

	// create a player with this name
	hr = lpDirectPlay2A->CreatePlayer(&dpidPlayer, &dpName, 
							lpDPInfo->hPlayerEvent, NULL, 0, 0);
	if FAILED(hr)
		goto CREATEPLAYER_FAILURE;

	// return connection info
	lpDPInfo->lpDirectPlay2A = lpDirectPlay2A;
	lpDPInfo->dpidPlayer = dpidPlayer;
	lpDPInfo->bIsHost = FALSE;

	return (DP_OK);

CREATEPLAYER_FAILURE:
OPEN_FAILURE:
	lpDirectPlay2A->Close();
	return (hr);
}

BOOL FAR PASCAL EnumSessionsCallback(
						LPCDPSESSIONDESC2	lpSessionDesc,
						LPDWORD				lpdwTimeOut,
						DWORD				dwFlags,
						LPVOID				lpContext)
{
	HWND			hWnd = (HWND) lpContext;
	LPGUID			lpGuid;
	LONG			iIndex;

	// see if enum has timed out
    if (dwFlags & DPESC_TIMEDOUT)
		return (FALSE);						// don't try again

	// store session name in list
	iIndex = SendDlgItemMessage(hWnd, IDC_SESSIONLIST, LB_ADDSTRING, 
								(WPARAM) 0, (LPARAM) lpSessionDesc->lpszSessionNameA);
	if (iIndex == CB_ERR)
		goto FAILURE;

	// make space for session instance guid
	lpGuid = (LPGUID) GlobalAllocPtr(GHND, sizeof(GUID));
	if (lpGuid == NULL)
		goto FAILURE;

	// store pointer to guid in list
	*lpGuid = lpSessionDesc->guidInstance;
	SendDlgItemMessage(hWnd, IDC_SESSIONLIST, LB_SETITEMDATA, (WPARAM) iIndex, (LPARAM) lpGuid);

FAILURE:
    return (TRUE);
}

HRESULT EnumSessions(HWND hWnd, LPDIRECTPLAY2A lpDirectPlay2A)
{
	DPSESSIONDESC2	sessionDesc;
	LONG			iIndex;
	HRESULT			hr;

	// check for valid interface
	if (lpDirectPlay2A == NULL)
		return (DPERR_INVALIDOBJECT);

	// delete existing list
	DeleteSessionInstanceList(hWnd);

	// search for this kind of session
	ZeroMemory(&sessionDesc, sizeof(DPSESSIONDESC2));
	sessionDesc.dwSize = sizeof(DPSESSIONDESC2);
    sessionDesc.guidApplication = DPCHAT_GUID;

	hr = lpDirectPlay2A->EnumSessions(&sessionDesc, 0, EnumSessionsCallback,
									  hWnd, DPENUMSESSIONS_AVAILABLE);
	if FAILED(hr)
		goto FAILURE;

	// select first item in list
	SendDlgItemMessage(hWnd, IDC_SESSIONLIST, LB_SETCURSEL,
						   (WPARAM) 0, (LPARAM) 0);

	// hilite "Join" button if there are sessions to join
	iIndex = SendDlgItemMessage(hWnd, IDC_SESSIONLIST, LB_GETCOUNT,
						   (WPARAM) 0, (LPARAM) 0);

	EnableDlgButton(hWnd, IDC_JOINBUTTON, (iIndex == 0) ? FALSE : TRUE);

FAILURE:
	return (hr);
}

HRESULT GetServiceProviderGuid(HWND hWnd, LPGUID lpguidServiceProvider)
{
	LONG	iIndex;

	// get guid for service provider to use for the connection
	iIndex = SendDlgItemMessage(hWnd, IDC_SPCOMBO, CB_GETCURSEL,
								(WPARAM) 0, (LPARAM) 0);
	if (iIndex == CB_ERR)
		return (DPERR_GENERIC);

	iIndex = SendDlgItemMessage(hWnd, IDC_SPCOMBO, CB_GETITEMDATA,
								(WPARAM) iIndex, (LPARAM) 0);
	if ((iIndex == CB_ERR) || (iIndex == 0))
		return (DPERR_GENERIC);

	*lpguidServiceProvider = *((LPGUID) iIndex);

	return (DP_OK);
}

void DeleteServiceProviderList(HWND hWnd)
{
	WPARAM	i;
	LONG	lpData;
	
	// destroy the GUID's stored with each service provider name
	i = 0;
	while (TRUE)
	{
		// get data pointer stored with item
		lpData = SendDlgItemMessage(hWnd, IDC_SPCOMBO, CB_GETITEMDATA,
									(WPARAM) i, (LPARAM) 0);
		if (lpData == CB_ERR)		// error getting data
			break;

		if (lpData == 0)			// no data to delete
			continue;

		GlobalFreePtr((LPVOID) lpData);
		i += 1;
	}

	// delete all items in combo box
	SendDlgItemMessage(hWnd, IDC_SPCOMBO, CB_RESETCONTENT,
								(WPARAM) 0, (LPARAM) 0);
}

HRESULT GetSessionInstanceGuid(HWND hWnd, LPGUID lpguidSessionInstance)
{
	LONG	iIndex;

	// get guid for session
	iIndex = SendDlgItemMessage(hWnd, IDC_SESSIONLIST, LB_GETCURSEL,
								(WPARAM) 0, (LPARAM) 0);
	if (iIndex == LB_ERR)
		return (DPERR_GENERIC);

	iIndex = SendDlgItemMessage(hWnd, IDC_SESSIONLIST, LB_GETITEMDATA,
								(WPARAM) iIndex, (LPARAM) 0);
	if ((iIndex == LB_ERR) || (iIndex == 0))
		return (DPERR_GENERIC);

	*lpguidSessionInstance = *((LPGUID) iIndex);

	return (DP_OK);
}

void DeleteSessionInstanceList(HWND hWnd)
{
	WPARAM	i;
	LONG	lpData;
	
	// destroy the GUID's stored with each session name
	i = 0;
	while (TRUE)
	{
		// get data pointer stored with item
		lpData = SendDlgItemMessage(hWnd, IDC_SESSIONLIST, LB_GETITEMDATA,
									(WPARAM) i, (LPARAM) 0);
		if (lpData == CB_ERR)		// error getting data
			break;

		if (lpData == 0)			// no data to delete
			continue;

		GlobalFreePtr((LPVOID) lpData);
		i += 1;
	}

	// delete all items in list
	SendDlgItemMessage(hWnd, IDC_SESSIONLIST, LB_RESETCONTENT,
								(WPARAM) 0, (LPARAM) 0);
}

void EnableDlgButton(HWND hDlg, int nIDDlgItem, BOOL bEnable)
{
	EnableWindow(GetDlgItem(hDlg, nIDDlgItem), bEnable);
}
